import os, sys
import time
import pwd
import grp
import subprocess as sp

merlin_conf_dir = "/opt/monitor/op5/merlin"

def initchildproc():
	os.umask(0377)

def cmd_generate(args):
	""" --path=</path/to/save/keys>
	Generates secure keys for encrypting Merlin communication

	By default the keys are placed in //opt/monitor/op5/merlin.
	"""

	path = merlin_conf_dir
	write_to_file = False
	for arg in args:
		if arg.startswith('--path'):
			path = arg.split('=', 1)[1]
		if arg.startswith('--write'):
			write_to_file = True

	has_pub_key = os.path.isfile(path + '/key.pub')
	has_priv_key = os.path.isfile(path + '/key.priv')

	if has_pub_key and has_priv_key:
		print 'File already exists. Keys will not be generated.'
		sys.exit(0)

	if has_pub_key or has_priv_key:
		print 'One of the encryption keys already exists. Remove and try again'
		sys.exit(1)

	if write_to_file:
		with open(merlin_conf_dir + "/merlin.conf", 'r') as fh:
			for line in fh:
				if "ipc_privatekey" in line:
					print "Conflict, private key location already specified: " + line.strip()
					sys.exit(1)

	p = sp.Popen(['/usr/lib64/merlin/keygen', "-p", path], stdout=sp.PIPE,
			stderr=sp.PIPE, preexec_fn=initchildproc)
	p.wait()
	if p.returncode != 0:
		print 'Key generation failed'
		sys.exit(1)

	if write_to_file:
		with open(merlin_conf_dir + "/merlin.conf", 'a') as fh:
			fh.write("ipc_privatekey = {}".format(path + "/key.priv"))
			fh.write('\n')

	uid = pwd.getpwnam("monitor").pw_uid
	# -1 means we leave group unchanged
	os.chown(path + "/key.priv", uid, -1)
	os.chown(path + "/key.pub", uid, -1)
	print 'Key generated and saved at ' + path
